/******************************************************************************
 Name:         EListPerson.cpp
 Version:      1.0
 Description:  
 History:      08/02/1999  Jérôme De Solliers   Creation
 ******************************************************************************/

#ifndef E_LIST_PERSON_H
#include "EListPerson.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <PopUpMenu.h>
#include <MenuItem.h>
#include <Messenger.h>


/******************************************************************************
 EListPerson::EListPerson()
 ******************************************************************************/
EListPerson::EListPerson()
{
	char *name_path = (char*) malloc(sizeof(char) * (B_PATH_NAME_LENGTH+B_FILE_NAME_LENGTH));
	memset(name_path, 0, (B_PATH_NAME_LENGTH+B_FILE_NAME_LENGTH));
	
	if (find_directory(B_USER_DIRECTORY, (dev_t)NULL, false, 
	                   name_path, (int32)strlen(name_path)) == B_OK )
	{
		m_name_path = name_path;
		BEntry	dir;
		dir.SetTo(m_name_path.String());
		
		m_name_path += "/people";
		
		m_init_status = B_OK;
	}
	else
		m_init_status = B_NO_INIT;
	
	delete name_path;
	
	m_people_list = new BList();
}


/******************************************************************************
 EListPerson::~EListPerson()
 ******************************************************************************/
EListPerson::~EListPerson()
{
	ClearList();
	delete m_people_list;
}


/******************************************************************************
 EListPerson::InitCheck()
 ******************************************************************************/
status_t EListPerson::InitCheck(void)
{
	return m_init_status;
}


/******************************************************************************
 EListPerson::CreateList()
 ******************************************************************************/
BList* EListPerson::CreateList(void)
{
	BDirectory	dir(m_name_path.String());
	BEntry		entry;
	char		temp[256];
	
	// Some clean up...
	ClearList();
	
	// Iterate through the people directory for searching the files
	dir.Rewind();
	while (dir.GetNextEntry(&entry) == B_OK)
	{
		BNode node(&entry);
		if (node.InitCheck() == B_OK)
		{
			if ( node.ReadAttr("META:name", (type_code)NULL, (off_t)NULL, (void *)temp, (size_t)sizeof(temp)) != B_ENTRY_NOT_FOUND )
			{
				people_info_list *item = new people_info_list;
				item->name = temp;
				
				if ( node.ReadAttr("META:email", (type_code)NULL, (off_t)NULL, (void *)temp, (size_t)sizeof(temp)) == B_ENTRY_NOT_FOUND )
					item->email = "";
				else
					item->email = temp;
				
				if ( node.ReadAttr("META:group", (type_code)NULL, (off_t)NULL, (void *)temp, (size_t)sizeof(temp)) == B_ENTRY_NOT_FOUND )
					item->group = "";
				else
					item->group = temp;
				
				entry.GetPath(&item->path);
				
				m_people_list->AddItem((void *)item);
			}
		}
	}
	
	m_people_list->SortItems(compare_string);
	return m_people_list;
}


/******************************************************************************
 EListPerson::ClearList()
 ******************************************************************************/
void EListPerson::ClearList(void)
{
	void	*item;
	
	for (int32 cpt=0 ; (item = m_people_list->ItemAt(cpt)) ; cpt++)
	{
		delete item;
//		delete (people_info_list *)item;
	}
	m_people_list->MakeEmpty();
}


/******************************************************************************
 EListPerson::BuildPopUpMenu()
 ******************************************************************************/
BPopUpMenu * EListPerson::BuildPopUpMenu(BPopUpMenu *menu, BMessenger *messenger)
{
	if ( InitCheck() != B_OK )
		return NULL;
	
	ExtMenuItem			*item;
	BMenuItem			*subitem;
	BMenu				*submenu = NULL;
	people_info_list	*listItem;
	uint32				cpt, cpt_item = m_people_list->CountItems();
	
	// Initialization of a group buffer
	BString		buffer = "0";
	
	// On scanne la liste en prenant en compte la notion de groupe
	cpt = 0;
	
	while ( cpt < cpt_item )
	{
		listItem = (people_info_list *)m_people_list->ItemAt(cpt);
		// If it's a new group, create a submenu
		if ( listItem->group != buffer )
		{
			buffer = listItem->group;
			
			if ( buffer.Length() )
			{
				// If it's a group
				submenu = new BMenu(buffer.String(), B_ITEMS_IN_COLUMN);
				subitem = new BMenuItem(submenu, NULL);
				menu->AddItem(subitem);
			}
			else
				// If it is a person not included in any group
				submenu = menu;
		}
		
		// Add the item and assign his email in the message
		if ( strlen(listItem->email.String()) == 0 )
			item = new ExtMenuItem(listItem->name.String(), new BMessage(E_MSG_WHAT_PERSON), false);
		else
			item = new ExtMenuItem(listItem->name.String(), new BMessage(E_MSG_WHAT_PERSON), true);
		item->Message()->AddString("path", listItem->path.Path());
		item->Message()->AddString("name", listItem->name.String());
		item->Message()->AddString("group", listItem->group.String());
		item->Message()->AddString("email", listItem->email.String());
		item->SetTarget(*messenger);
		submenu->AddItem(item);
		
		cpt++;
	}
	
	return menu;
}


//******************************************************************************
// compare_string()
// ******************************************************************************/
int compare_string(const void *item1, const void *item2)
{
	people_info_list *people1 = (people_info_list *)((BList *)item1)->Items();
	people_info_list *people2 = (people_info_list *)((BList *)item2)->Items();
	
	return strcasecmp(people1->group.String(), people2->group.String());
}


//*****************************************************************************
// ExtMenuItem::ExtMenuItem()
//*****************************************************************************
ExtMenuItem::ExtMenuItem(const char *label, BMessage *message, bool email, char shortcut, uint32 modifiers)
           : BMenuItem(label, message, shortcut, modifiers)
{
	m_email = email;
}

ExtMenuItem::ExtMenuItem(BMenu *submenu, BMessage *message)
           : BMenuItem(submenu, message)
{
	m_email = false;
}


//*****************************************************************************
// ExtMenuItem::Invoke()
//*****************************************************************************
status_t ExtMenuItem::Invoke(BMessage *message)
{
	bool	email;
	
	if ( modifiers() & B_SHIFT_KEY )
		email = true;
	else
		email = false;
	
	message = Message();
	BMessage sendMsg(*message);
	sendMsg.AddBool("email_selected", email);
	
	return BMenuItem::Invoke(&sendMsg);
}


//*****************************************************************************
// ExtMenuItem::DrawContent()
//*****************************************************************************
void ExtMenuItem::DrawContent(void)
{
	BMenu *menu = Menu();
	rgb_color	color;
	color = menu->HighColor();
	if ( !m_email )
		menu->SetHighColor(255,0,0,0);
	BMenuItem::DrawContent();
	if ( !m_email )
		menu->SetHighColor(color);
}

