/******************************************************************************
 Name:         QLPAbout.cpp
 Version:      1.3
 Description:  
 History:      25/06/2001  Jérôme De Solliers   Creation
 ******************************************************************************/

#include "QLPAbout.h"
#include "BackgroundView.h"
#include "Bitmaps.h"
#include <Button.h>
#include <TextView.h>
#include <Screen.h>


//*****************************************************************************
// QLPAbout::QLPAbout()
//*****************************************************************************
QLPAbout::QLPAbout(const char *title, BView *view)
		: BWindow(BRect(100,100,399,299), title, B_MODAL_WINDOW_LOOK, 
		          B_NORMAL_WINDOW_FEEL, B_NOT_RESIZABLE)
{
	m_QLPView = view;
	_BuildInterface();
	CenterMe();
}


//*****************************************************************************
// QLPAbout::~QLPAbout()
//*****************************************************************************
QLPAbout::~QLPAbout()
{
	delete m_iconBitmap;
}


//*****************************************************************************
// QLPAbout::_BuildInterface()
//*****************************************************************************
void QLPAbout::_BuildInterface()
{
	// Set the style for the text view
	BFont font1(be_plain_font);
	BFont font2(be_bold_font);
	rgb_color	color = {0,0,0,0};
	text_run			m_runs_normal[1];
	text_run_array		m_run_array_normal;
	text_run			m_runs_title[1];
	text_run_array		m_run_array_title;
	
	m_runs_normal[0].offset = 0;
	m_runs_normal[0].font = font1;
	m_runs_normal[0].color = color;
	m_run_array_normal.count = 1;
	m_run_array_normal.runs = m_runs_normal;
	
	m_runs_title[0].offset = 0;
	m_runs_title[0].font = font2;
	m_runs_title[0].color = color;
	m_run_array_title.count = 1;
	m_run_array_title.runs = m_runs_title;
	
	// Create the icon bitmap
	m_iconBitmap = new BBitmap(BRect(0, 0, B_LARGE_ICON-1, B_LARGE_ICON-1), B_CMAP8,
	                                 false, false);
	m_iconBitmap->SetBits(kLargeIconBits, B_LARGE_ICON*B_LARGE_ICON, 0, B_CMAP8);
	
	// Add the top view
	BRect rect = Bounds();
	BackgroundView *topView = new BackgroundView(rect, "", m_iconBitmap, B_FOLLOW_ALL, B_WILL_DRAW);
	AddChild(topView);
	
	// Add the text view
	BRect rect2;
	rect.Set(50, Bounds().top+10, Bounds().right-10, Bounds().bottom-40);
	rect2.Set(0, 0, rect.Width(), rect.Height());
	BTextView *txtAbout = new BTextView(rect, "", rect2, B_FOLLOW_ALL, B_WILL_DRAW);
	topView->AddChild(txtAbout);
	txtAbout->SetViewColor(216,216,216,0);
	txtAbout->MakeEditable(false);
	txtAbout->MakeSelectable(false);
	txtAbout->SetStylable(true);
	
	txtAbout->SetText(TEXT_ABOUT2, &m_run_array_normal);
	txtAbout->Insert(TEXT_ABOUT1, &m_run_array_title);
	
	// Add the OK button
	rect.Set(Bounds().right-80, Bounds().bottom-30, Bounds().right-10, Bounds().bottom-10);
	BButton *btnOk = new BButton(rect, "", "OK", new BMessage(B_QUIT_REQUESTED));
	topView->AddChild(btnOk);
	SetDefaultButton(btnOk);
}


//*****************************************************************************
// QLPAbout::QuitRequested()
//*****************************************************************************
bool QLPAbout::QuitRequested(void)
{
	((QLPDeskbarView *)m_QLPView)->m_aboutCurrentlyDisplay = false;
	
	return true;
}


//*****************************************************************************
// QLPAbout::CenterMe()
//*****************************************************************************
void QLPAbout::CenterMe(void)
{
	float		width, height;
	
	BScreen *screen = new BScreen(B_MAIN_SCREEN_ID);
	BRect srect = screen->Frame();
	delete screen;
	
	width  = Frame().right - Frame().left;
	height = Frame().bottom - Frame().top;
	
	MoveTo((((srect.right - srect.left) - width) / 2), 
	       (((srect.bottom - srect.top) - height) / 2));
}

