/******************************************************************************
 Name:         QuickLaunchPeople.cpp
 Version:      1.3
 Description:  
 History:      09/06/2001  Jérôme De Solliers   Creation
 ******************************************************************************/

#include "QuickLaunchPeople.h"
#include "Bitmaps.h"


//*****************************************************************************
// main()
//*****************************************************************************
int main()
{
	BApplication app(APP_SIGNATURE);
	
	LoadDeskbar();
	
	return 0;
}


//*****************************************************************************
// New system for adding an addon in the Deskbar
//*****************************************************************************
extern "C" _EXPORT BView* instantiate_deskbar_item();

BView * instantiate_deskbar_item()
{
	return new QLPDeskbarView();
}


//*****************************************************************************
// LoadDeskbar() : Tell the deskbar to install the view
//*****************************************************************************
bool LoadDeskbar(void)
{
	//  Check the deskbar to see if the replicant already exist...
	BDeskbar *deskbar = new BDeskbar();
	if ( deskbar->HasItem(DESKBAR_VIEW_NAME) )
	{
		delete deskbar;
		return false;
	}
	
	BRoster roster;
	entry_ref ref;
	roster.FindApp(APP_SIGNATURE, &ref);
	int32 id;
	status_t err = deskbar->AddItem(&ref, &id);
	delete deskbar;
	
	if ( err == B_OK )
		return true;
	else 
		return false;
}


//*****************************************************************************
// QLPDeskbarView::QLPDeskbarView()
//*****************************************************************************
QLPDeskbarView::QLPDeskbarView()
                 : BView(BRect(0, 0, B_MINI_ICON-1, B_MINI_ICON-1), 
                         DESKBAR_VIEW_NAME, B_FOLLOW_LEFT | B_FOLLOW_TOP,
                         B_WILL_DRAW)
{
	Init();
}


QLPDeskbarView::QLPDeskbarView(BMessage *msg)
                  : BView(msg)
{
	Init();
}


//*****************************************************************************
// QLPDeskbarView::Init()
//*****************************************************************************
void QLPDeskbarView::Init(void)
{
	m_listPerson = NULL;
	m_iconBitmap = new BBitmap(BRect(0, 0, B_MINI_ICON-1, B_MINI_ICON-1), B_CMAP8,
	                                 false, false);
	m_iconBitmap->SetBits(kSmallIconBits, B_MINI_ICON*B_MINI_ICON, 0, B_CMAP8);
	m_aboutCurrentlyDisplay = false;
}


//*****************************************************************************
// QLPDeskbarView::Archive()
//*****************************************************************************
status_t QLPDeskbarView::Archive(BMessage *msg, bool deep) const
{
	BView::Archive(msg, deep);
	msg->AddString("add_on", APP_SIGNATURE);
	msg->AddString("class", DESKBAR_VIEW_NAME);
	
	return B_OK;
}


//*****************************************************************************
// QLPDeskbarView::Instantiate()
//*****************************************************************************
QLPDeskbarView *QLPDeskbarView::Instantiate(BMessage *msg)
{
	if ( !validate_instantiation(msg, DESKBAR_VIEW_NAME) )
		return NULL;
	
	return new QLPDeskbarView(msg);
}


//*****************************************************************************
// QLPDeskbarView::AttachedToWindow()
//*****************************************************************************
void QLPDeskbarView::AttachedToWindow(void)
{
	
	if ( Parent() )
		SetViewColor(Parent()->ViewColor());
	
	// Now, build the pop up menu list
	BuildPopUpList();
}


//*****************************************************************************
// QLPDeskbarView::BuildPopUpList()
//*****************************************************************************
void QLPDeskbarView::BuildPopUpList(void)
{
	// Install the popup menu
	BMessenger messenger(this);
	
	DeskBarMenu = new BPopUpMenu("DeskBarMenu", false, false, B_ITEMS_IN_COLUMN);
	
	// Add the people list in the popupmenu
	if ( !m_listPerson )
		m_listPerson = new EListPerson();
	else
		m_listPerson->ClearList();
	
	if ( m_listPerson->InitCheck() == B_OK )
	{
		m_listPerson->CreateList();
		m_listPerson->BuildPopUpMenu(DeskBarMenu, &messenger);
	}
	DeskBarMenu->SetFont(be_plain_font);
	
	DeskBarMenu->AddSeparatorItem();
	
	BMenuItem	*item_reload = new BMenuItem("Reload the list", new BMessage(MSG_RELOAD));
	item_reload->SetTarget(messenger);
	DeskBarMenu->AddItem(item_reload);
	
	BMenuItem	*item_create = new BMenuItem("Create a new person", new BMessage(MSG_CREATE));
	item_create->SetTarget(messenger);
	DeskBarMenu->AddItem(item_create);
	
	BMenuItem	*item_folder = new BMenuItem("Open the people folder", new BMessage(MSG_OPEN_DIRECTORY));
	item_folder->SetTarget(messenger);
	DeskBarMenu->AddItem(item_folder);
	
	BMenuItem	*item_about = new BMenuItem("About...", new BMessage(MSG_ABOUT));
	item_about->SetTarget(messenger);
	DeskBarMenu->AddItem(item_about);
	
	DeskBarMenu->AddSeparatorItem();
	
	BMenuItem	*item_quit = new BMenuItem("Quit", new BMessage(MSG_QUIT));
	DeskBarMenu->AddItem(item_quit);
	item_quit->SetTarget(messenger);
}


//*****************************************************************************
// QLPDeskbarView::DetachedFromWindow()
//*****************************************************************************
void QLPDeskbarView::DetachedFromWindow(void)
{
	delete m_iconBitmap;
}


//*****************************************************************************
// QLPDeskbarView::Draw()
//*****************************************************************************
void QLPDeskbarView::Draw(BRect updateRect)
{
	if ( m_iconBitmap )
	{
		SetDrawingMode(B_OP_OVER);
		DrawBitmap(m_iconBitmap, BPoint(0,0));
	}
}


//*****************************************************************************
// QLPDeskbarView::MouseDown()
//*****************************************************************************
void QLPDeskbarView::MouseDown(BPoint where) 
{
	ulong	buttons;

	GetMouse(&where, &buttons);
	where.y--;
	ConvertToScreen(&where);
	
//	if ( buttons == B_SECONDARY_MOUSE_BUTTON )
//	{
		if ( DeskBarMenu )
			DeskBarMenu->Go(where, true, false, true);
//	}
}


//*****************************************************************************
// QLPDeskbarView::MessageReceived()
//*****************************************************************************
void QLPDeskbarView::MessageReceived(BMessage *msg)
{
	switch ( msg->what )
	{
		// Open the person file
		case E_MSG_WHAT_PERSON :
		{
			bool email = false;
			msg->FindBool("email_selected", &email);
			
			if ( !email )
			{
				// Launch people software
				char	*buffer = (char *)msg->FindString("path");
				
				entry_ref	ref;
				get_ref_for_path(buffer, &ref);
				be_roster->Launch((const entry_ref *)&ref);
			}
			else
			{
				// Launch the email software
				char *strTemp = (char *)msg->FindString("email");
				if ( strlen(strTemp) == 0 )
				{
					BAlert *alert = new BAlert("QLP Notify", "I can't launch your preferred email client because this person doesn't have any email address.", "OK");
					alert->SetShortcut(0, B_ESCAPE);
					alert->Go(NULL);
				}
				else
				{
					char *buffer = new char[8+strlen(strTemp)+1];
					strcpy(buffer, "mailto:");
					strcat(buffer, strTemp);
					const char *argv[1];
					argv[0] = buffer;
					be_roster->Launch(EMAIL_SIGNATURE, 1, (char **)argv);
					delete[] buffer;
				}
			}
		}
		break;
		
		// Reload the people list
		case MSG_RELOAD :
		{
			delete DeskBarMenu;
			DeskBarMenu = NULL;
			BuildPopUpList();
		}
		break;
		
		// Create a new person
		case MSG_CREATE :
		{
			CreateNewPerson();
		}
		break;
		
		// Open the people directory
		case MSG_OPEN_DIRECTORY :
		{
			OpenDirectory();
		}
		break;
		
		// About message
		case MSG_ABOUT :
		{
			if ( !m_aboutCurrentlyDisplay )
			{
				m_winAbout = new QLPAbout("About", this);
				m_winAbout->Show();
				m_aboutCurrentlyDisplay = true;
			}
			else
				m_winAbout->Activate(true);
		}
		break;
		
		// Quit
		case MSG_QUIT :
		{
			_RemoveFromDeskbar();
		}
		break;
		
		default:
			BView::MessageReceived(msg);
		break;
	}
}


//*****************************************************************************
// QLPDeskbarView::CreateNewPerson()
//*****************************************************************************
bool QLPDeskbarView::CreateNewPerson(void)
{
	bool	retVal = false;
	
	BMimeType	*mime = new BMimeType(PERSON_SIGNATURE);
	if ( mime->InitCheck() == B_OK )
	{
		char app_signature[B_MIME_TYPE_LENGTH+1];
		if ( mime->GetPreferredApp(app_signature) == B_OK )
		{
			if ( be_roster->Launch(app_signature) == B_OK )
				retVal = true;
		}
	}
	delete mime;
	
	return retVal;
}


/******************************************************************************
 QLPDeskbarView::OpenDirectory()
 Ask the Tracker to open on screen the directory
 ******************************************************************************/
void QLPDeskbarView::OpenDirectory(void)
{
	BEntry		entry;
	BDirectory	dir;
	
	dir.SetTo(PEOPLE_FOLDER); 	//setup default directory
	
	if ( dir.GetEntry(&entry) == B_OK )
	{
		entry_ref	ref;
		entry.GetRef(&ref);
		
		BMessage	message(B_REFS_RECEIVED);
		BMessenger	*tracker = new BMessenger(TRACKER_SIGNATURE, -1, NULL);
		if ( tracker->IsValid() )
		{
			message.AddRef("refs", &ref);
			tracker->SendMessage(&message);
		}
		delete tracker;
	}
}


/******************************************************************************
 QLPDeskbarView::_RemoveFromDeskbar()
 ******************************************************************************/
void QLPDeskbarView::_RemoveFromDeskbar(void)
{
	BDeskbar *deskbar = new BDeskbar();
	status_t err = deskbar->RemoveItem(DESKBAR_VIEW_NAME);
	if ( err != B_OK )
	{
		BAlert *alert = new BAlert("Info", "Can't remove the replicant from the deskbar. Sorry.", "OK");
		alert->Go(NULL);
	}
	delete deskbar;
}
